/*
 * For each possible key length and password letter position,
 * test each letter as a candidate
 * and compare the frequency distribution with that of English.
 */

#include <stdio.h>
#include <math.h>

#define MAXTEXT	(16 * 1024)
#define MAXKEY	32

void	Test (char text[], int textsize, int keylen, int keypos, char candidate);

int	main ()
{
	char	text[MAXTEXT];
	char	*from, *to, letter;
	int	textsize, keylen, keypos;

	if ((textsize = fread (text, 1, MAXTEXT, stdin)) <= 0)
		return 1;

	// test

	for (keylen = 1;   keylen <= MAXKEY; keylen++)
	for (keypos = 0;   keypos <  keylen; keypos++)
	for (letter = 'A'; letter <= 'Z'   ; letter++)
		Test (text, textsize, keylen, keypos, letter);

	return 0;
}

#define LETTERS	('Z' - 'A' + 1)

double	englishFreq[LETTERS] = {
	8.167,
	1.492,
	2.782,
	4.253,
	12.702,
	2.228,
	2.015,
	6.094,
	6.996,
	0.153,
	0.772,
	4.025,
	2.406,
	6.749,
	7.507,
	1.929,
	0.095,
	5.987,
	6.327,
	9.056,
	2.758,
	0.978,
	2.360,
	0.150,
	1.974,
	0.074
};

void	Test (char text[], int textsize, int keylen, int keypos, char candidate)
{
	int	i, n;
	double	histo[LETTERS];
	double	sum, avg, d, var;

	for (i = 0; i < LETTERS; i++)
		histo[i] = 0;

	for (n = 0, i = keypos; i < textsize; i += keylen, n++)
		if (text[i] >= 'A' && text[i] <= 'Z')
			histo[(text[i] - candidate + LETTERS) % LETTERS]++;

	for (i = 0; i < LETTERS; i++)
		if ((histo[i] = 100. * histo[i] / n - englishFreq[i]) < 0)
			histo[i] = -histo[i];

	sum = 0;
	for (i = 0; i < LETTERS; i++)
		sum += histo[i];
	avg = sum / LETTERS;

	sum = 0;
	for (i = 0; i < LETTERS; i++) {
		d = histo[i] - avg;
		sum += d * d;
	}
	var = sum / LETTERS;

	printf ("%c %d %d\t%.5f\t%.5f\n", candidate, keypos, keylen, avg, sqrt (var));
}
